# -*- coding: utf-8 -*-
from DateTime import DateTime
from pkg_resources import get_distribution
from pkg_resources import parse_version
from plone.app.event.base import _get_compare_attr
from plone.app.event.base import _obj_or_acc
from plone.app.event.base import RET_MODE_BRAINS
from plone.app.event.dx.behaviors import EventAccessor
from plone.app.event.recurrence import EventOccurrenceAccessor
from plone.app.querystring import queryparser
from plone.base.interfaces import IImageScalesAdapter
from plone.event.interfaces import IEvent
from plone.event.interfaces import IEventRecurrence
from plone.event.interfaces import IRecurrenceSupport
from plone.restapi.deserializer import json_body
from plone.restapi.services import Service
from Products.CMFPlone.interfaces import IPloneSiteRoot
from Products.CMFPlone.utils import safe_hasattr
from zope.component import getMultiAdapter
from zope.component import queryMultiAdapter


zcatalog_version = get_distribution("Products.ZCatalog").version
if parse_version(zcatalog_version) >= parse_version("5.1"):
    SUPPORT_NOT_UUID_QUERIES = True
else:
    SUPPORT_NOT_UUID_QUERIES = False


class BaseService(Service):
    def expand_events(
        self, events, ret_mode, start=None, end=None, sort=None, sort_reverse=None
    ):
        """Expand to the recurrence occurrences of a given set of events.

        :param events: IEvent based objects or IEventAccessor object wrapper.

        :param ret_mode: Return type of search results. These options are
                        available:

                            * 2 (objects): Return results as IEvent and/or
                                            IOccurrence objects.
                            * 3 (accessors): Return results as IEventAccessor
                                            wrapper objects.
                        Option "1" (brains) is not supported.

        :type ret_mode: integer [2|3]

        :param start: Date, from which on events should be expanded.
        :type start: Python datetime.

        :param end: Date, until which events should be expanded.
        :type end: Python datetime

        :param sort: Object or IEventAccessor Attribute to sort on.
        :type sort: string

        :param sort_reverse: Change the order of the sorting.
        :type sort_reverse: boolean

        COPIATO DA plone.app.event 3.2.10 perché nella 3.2.13 è cambiato e si
        rompe con i parametri che gli passiamo. Visto che la logica ci andava
        bene così, non stiamo a metterci mano.

        """

        assert ret_mode is not RET_MODE_BRAINS  # nosec

        exp_result = []
        for it in events:
            obj = it.getObject() if getattr(it, "getObject", False) else it
            if IEventRecurrence.providedBy(obj):
                occurrences = [
                    _obj_or_acc(occ, ret_mode)
                    for occ in IRecurrenceSupport(obj).occurrences(start, end)
                ]
            elif IEvent.providedBy(obj):
                occurrences = [_obj_or_acc(obj, ret_mode)]
            else:
                # No IEvent based object. Could come from a collection.
                continue
            exp_result += occurrences
        if sort:
            exp_result.sort(key=lambda x: _get_compare_attr(x, sort))
        if sort_reverse:
            exp_result.reverse()
        return exp_result


class ScadenziarioSearchPost(BaseService):
    """
    Ritorna la lista dei giorni in cui sono presenti elementi da visualizzare
    {
        "@id": "http://localhost:9080/Plone/@scadenziario",
        "items": [
            "2020/11/19",
            "2020/11/26",
            "2020/12/17",
            "2020/12/24",
            "2020/12/31",
            "2021/01/07"
        ]
    }
    """

    def reply(self):
        data = json_body(self.request)
        query = data.get("query", None)
        # b_start = int(data.get("b_start", 0))
        # b_size = int(data.get("b_size", 25))
        sort_on = data.get("sort_on", None)
        sort_order = data.get("sort_order", None)
        limit = int(data.get("limit", 1000))
        # fullobjects = data.get("fullobjects", False)

        if query is None:
            raise Exception("No query supplied")

        if sort_order:
            sort_order = "descending" if sort_order == "descending" else "ascending"

        querybuilder = getMultiAdapter(
            (self.context, self.request), name="querybuilderresults"
        )
        querybuilder_parameters = dict(
            query=query,
            brains=True,
            # b_start=b_start,
            # b_size=b_size,
            sort_on=sort_on,
            sort_order=sort_order,
            limit=limit,
        )

        # Exclude "self" content item from the results when ZCatalog supports
        # NOT UUID
        # queries and it is called on a content object.
        if not IPloneSiteRoot.providedBy(self.context) and SUPPORT_NOT_UUID_QUERIES:
            querybuilder_parameters.update(
                dict(custom_query={"UID": {"not": self.context.UID()}})
            )

        # Ottieni tutti i risultati
        results = querybuilder(**querybuilder_parameters)

        # preparati per l'expand degli eventi.
        not_events = [x for x in results if x.portal_type != "Event"]
        events = [x for x in results if x.portal_type == "Event"]
        # prende la query e la trasforma in una query per il catalogo
        # così poi se e quando dobbiamo litigare con delle ricorrenze e date
        # di start ed end, le abbiamo già calcolate, come plone le proporrebbe
        # al catalogo
        query_for_catalog = queryparser.parseFormquery(
            self.context, query, sort_on=sort_on, sort_order=sort_order
        )
        start = None
        end = None
        if "start" in query_for_catalog:
            start = query_for_catalog["start"]["query"]
        if "end" in query_for_catalog:
            if query_for_catalog["end"].get("range", "") != "min":
                # per esempio, è impostato il filtro "con fine evento da domani".
                # se impostiamo un'end (la data  di domani), poi nella generazione delle ricorrenze,
                # vengono scartati tutti gli eventi che hanno una data di inizio nel futuro
                # (https://github.com/plone/plone.event/blob/master/plone/event/recurrence.py#L141)
                # perché la data della ricorrenza è maggiore di "until", che è quello che qui inviamo come end.
                end = query_for_catalog["end"]["query"]
        expanded_events = self.expand_events(events, 3, start, end)

        all_results = not_events + expanded_events
        brains_grouped = {}
        for brain in all_results:
            if not safe_hasattr(brain, "start") or not brain.start:
                continue
            brains_grouped.setdefault(brain.start.strftime("%Y/%m/%d"), []).append(
                brain
            )
        keys = list(brains_grouped.keys())
        if sort_order == "descending":
            keys.sort(reverse=True)
        else:
            keys.sort()

        return {"@id": self.request.get("URL"), "items": keys}


class ScadenziarioDayPost(BaseService):
    def reply(self):
        data = json_body(self.request)
        query = data.get("query", None)
        sort_on = data.get("sort_on", None)
        sort_order = data.get("sort_order", None)

        if query is None:
            raise Exception("No query supplied")

        if sort_order not in {"descending", "ascending"}:
            sort_order = "ascending"

        # results = querybuilder(**querybuilder_parameters)
        # Seems that origina querybuilder is not able to handle event search on
        # a single day... I can handle this calling catalog and going through
        # DateTime conversion
        query_for_catalog = queryparser.parseFormquery(
            self.context, query, sort_on=sort_on, sort_order=sort_order
        )
        query_for_catalog["start"]["query"][0] = DateTime(
            query_for_catalog["start"]["query"][0]
        )
        query_for_catalog["start"]["query"][1] = DateTime(
            query_for_catalog["start"]["query"][1]
        )
        results = self.context.portal_catalog(query_for_catalog)
        # preparati per l'expand degli eventi.
        not_events = [x for x in results if x.portal_type != "Event"]
        events = [x for x in results if x.portal_type == "Event"]
        start = None
        end = None
        # qui ce l'abbiamo per forza start
        if "start" in query_for_catalog:
            start = query_for_catalog["start"]["query"][0]
        if "end" in query_for_catalog:
            end = query_for_catalog["start"]["query"][1]

        expanded_events = self.expand_events(events, 3, start, end)
        start_date = start.strftime("%Y/%m/%d")
        correct_events = []
        for x in expanded_events:
            if start_date == x.start.strftime("%Y/%m/%d"):
                correct_events.append(x)

        all_results = not_events + correct_events

        brains_grouped = {}
        for brain in all_results:
            if not safe_hasattr(results[0], "start"):
                continue
            brains_grouped.setdefault(brain.start.strftime("%Y/%m/%d"), []).append(
                brain
            )

        keys = list(brains_grouped.keys())
        keys.sort()

        results_to_be_returned = {}
        for key in keys:
            results_to_be_returned[key] = []
            for brain in brains_grouped[key]:
                if isinstance(brain, (EventAccessor, EventOccurrenceAccessor)):
                    if brain.context.portal_type == "Occurrence":
                        url = brain.url[:-10]
                        scales = queryMultiAdapter(
                            (brain.context.aq_parent, self.request), IImageScalesAdapter
                        )
                        image_scales = scales()
                    else:
                        url = brain.url
                        scales = queryMultiAdapter(
                            (brain.context, self.request), IImageScalesAdapter
                        )
                        image_scales = scales()

                    results_to_be_returned[key].append(
                        {
                            "@id": url,
                            "id": brain.id,
                            "title": brain.title,
                            "text": brain.description,
                            "start": brain.start.isoformat(),
                            "type": self.context.translate("Event"),
                            "category": brain.subjects,
                            "image_scales": image_scales,
                        }
                    )

                else:
                    results_to_be_returned[key].append(
                        {
                            "@id": brain.getURL(),
                            "id": brain.getId,
                            "title": brain.Title,
                            "text": brain.Description,
                            "start": brain.start.isoformat(),
                            "type": self.context.translate(brain.portal_type),
                            "category": brain.subject,
                        }
                    )

        return {
            "@id": self.request.get("URL"),
            "items": results_to_be_returned,
        }



# -*- coding: utf-8 -*-
from datetime import datetime
from datetime import timedelta
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from transaction import commit

import unittest


class ScadenziarioTest(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        commit()

    def tearDown(self):
        self.api_session.close()

    def test_return_future_events_if_query_is_end_after_today(self):
        now = datetime.now()

        # past event
        api.content.create(
            container=self.portal,
            type="Event",
            title="Past event",
            start=now.replace(hour=8) + timedelta(days=-2),
            end=now.replace(hour=18) + timedelta(days=-2),
        )

        future_event_1 = api.content.create(
            container=self.portal,
            type="Event",
            title="Future event",
            start=now.replace(hour=8) + timedelta(days=2),
            end=now.replace(hour=18) + timedelta(days=4),
        )
        future_event_2 = api.content.create(
            container=self.portal,
            type="Event",
            title="Future event that starts in the past",
            start=now.replace(hour=8) + timedelta(days=-4),
            end=now.replace(hour=18) + timedelta(days=4),
        )

        commit()

        response = self.api_session.post(
            f"{self.portal_url}/@scadenziario",
            json={
                "query": [
                    {
                        "i": "end",
                        "o": "plone.app.querystring.operation.date.afterToday",
                        "v": "",
                    }
                ]
            },
        ).json()
        self.assertEqual(len(response["items"]), 2)

        # results are in asc order
        self.assertEqual(
            response["items"][0],
            future_event_2.start.strftime("%Y/%m/%d"),
        )
        self.assertEqual(
            response["items"][1],
            future_event_1.start.strftime("%Y/%m/%d"),
        )



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces import IDesignPloneContenttypesLayer
from design.plone.contenttypes.utils import create_default_blocks
from plone import api
from Products.CMFPlone.interfaces import ISelectableConstrainTypes


SUBFOLDERS_MAPPING = {
    "Bando": {
        "content": [
            {"id": "documenti", "title": "Documenti", "type": "Bando Folder Deepening"},
            {
                "id": "comunicazioni",
                "title": "Comunicazioni",
                "type": "Bando Folder Deepening",
            },
            {"id": "esiti", "title": "Esiti", "type": "Bando Folder Deepening"},
        ],
    },
    "Documento": {
        "content": [
            {
                "id": "multimedia",
                "title": "Multimedia",
                "type": "Document",
                "allowed_types": ("Image",),
            },
        ],
    },
    "Event": {
        "content": [
            {
                "id": "immagini",
                "title": "Immagini",
                "allowed_types": ("Image", "Link"),
                "publish": True,
            },
            {
                "id": "video",
                "title": "Video",
                "allowed_types": ("Link",),
                "publish": True,
            },
            {
                "id": "sponsor_evento",
                "title": "Sponsor Evento",
                "allowed_types": ("Link",),
                "publish": True,
            },
            {
                "id": "documenti",
                "title": "Allegati",
                "allowed_types": ("File",),
                "publish": True,
            },
        ],
    },
    "Incarico": {
        "content": [
            {"id": "compensi-file", "title": "Compensi", "allowed": ("File",)},
            {
                "id": "importi-di-viaggio-e-o-servizi",
                "title": "Importi di viaggio e/o servizi",
                "allowed_types": ("File",),
            },
        ],
        "allowed_types": [],
    },
    "Venue": {
        "content": [
            {
                "id": "multimedia",
                "title": "Multimedia",
                "type": "Folder",
                "allowed_types": (
                    "Image",
                    "Link",
                ),
                "publish": True,
            }
        ],
    },
    "News Item": {
        "content": [
            {
                "id": "multimedia",
                "title": "Multimedia",
                "allowed_types": (
                    "Image",
                    "Link",
                ),
            },
            {
                "id": "documenti-allegati",
                "title": "Documenti allegati",
                "allowed_types": (
                    "File",
                    "Image",
                ),
            },
        ],
    },
    "Persona": {
        "content": [
            {
                "id": "foto-e-attivita-politica",
                "title": "Foto e attività politica",
                "allowed_types": ("Image",),
            },
            {
                "id": "curriculum-vitae",
                "title": "Curriculum vitae",
                "allowed_types": ("File",),
            },
            {
                "id": "situazione-patrimoniale",
                "title": "Situazione patrimoniale",
                "allowed_types": ("File",),
            },
            {
                "id": "dichiarazione-dei-redditi",
                "title": "Dichiarazione dei redditi",
                "allowed_types": ("File",),
            },
            {
                "id": "spese-elettorali",
                "title": "Spese elettorali",
                "allowed_types": ("File",),
            },
            {
                "id": "variazione-situazione-patrimoniale",
                "title": "Variazione situazione patrimoniale",
                "allowed_types": ("File",),
            },
            {
                "id": "altre-cariche",
                "title": "Altre cariche",
                "allowed_types": ("File",),
            },
            {"id": "incarichi", "title": "Incarichi", "allowed_types": ("Incarico",)},
            {
                "id": "altri-documenti",
                "title": "Altri documenti",
                "allowed_types": ("File", "Image", "Link"),
            },
            {
                "id": "dichiarazione-insussistenza-cause-di-inconferibilita-e-incompatibilita",  # noqa
                "title": "Dichiarazione insussistenza cause di inconferibilità e"
                " incompatibilità",
                "allowed_types": ("File",),
            },
            {
                "id": "emolumenti-complessivi-percepiti-a-carico-della-finanza-pubblica",  # noqa
                "title": "Emolumenti complessivi percepiti a carico della finanza"
                " pubblica",
                "allowed_types": ("File",),
            },
        ],
        "allowed_types": [],
    },
    "Pratica": {
        "content": [
            {
                "id": "allegati",
                "title": "Allegati",
                "type": "Folder",
                "allowed_types": ("File",),
            }
        ],
    },
    "Servizio": {
        "content": [
            {
                "id": "modulistica",
                "title": "Modulistica",
                "allowed_types": ("Link",),
            },
            {"id": "allegati", "title": "Allegati", "allowed_types": ("File", "Link")},
        ],
    },
    "UnitaOrganizzativa": {
        "content": [
            {"id": "allegati", "title": "Allegati", "allowed_types": ("File",)},
        ],
    },
}


def onModify(context, event):
    for description in event.descriptions:
        if "IBasic.title" in getattr(
            description, "attributes", []
        ) or "IDublinCore.title" in getattr(description, "attributes", []):
            context_state = api.content.get_view(
                name="plone_context_state", context=context, request=context.REQUEST
            )
            if context_state.is_folderish():
                for child in context.listFolderContents():
                    child.reindexObject(idxs=["parent"])


def createStructure(context, subfolders_mapping):

    for mapping in subfolders_mapping.get("content", {}):
        if mapping["id"] not in context.keys():
            portal_type = mapping.get("type", "Document")
            child = api.content.create(
                container=context,
                type=portal_type,
                title=mapping["title"],
                id=mapping["id"],
            )
            if portal_type == "Document":
                create_default_blocks(context=child)

            if portal_type in ["Folder", "Document"]:
                child.exclude_from_search = True
                child.reindexObject(idxs=["exclude_from_search"])
            # select constraints
            if mapping.get("allowed_types", ()):
                constraints_child = ISelectableConstrainTypes(child)
                constraints_child.setConstrainTypesMode(1)
                constraints_child.setLocallyAllowedTypes(mapping["allowed_types"])

            if mapping.get("publish", False):
                with api.env.adopt_roles(["Reviewer"]):
                    api.content.transition(obj=child, transition="publish")

    allowed_types = subfolders_mapping.get("allowed_types", None)
    if allowed_types is not None and not isinstance(allowed_types, list):
        raise ValueError("Subfolder map is not well formed")

    if isinstance(allowed_types, list):
        constraints_context = ISelectableConstrainTypes(context)
        constraints_context.setConstrainTypesMode(1)
        constraints_context.setLocallyAllowedTypes(allowed_types)


def createSubfolders(context, event):
    """
    Create subfolders structure based on a portal_type mapping
    """
    if not IDesignPloneContenttypesLayer.providedBy(context.REQUEST):
        return

    subfolders_mapping = SUBFOLDERS_MAPPING.get(context.portal_type, [])
    if not subfolders_mapping:
        return
    createStructure(context, subfolders_mapping)



# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from design.plone.contenttypes.interfaces.bando import IBandoAgidSchema
from design.plone.contenttypes.interfaces.documento import IDocumento
from design.plone.contenttypes.interfaces.servizio import IServizio
from plone.app.contenttypes.interfaces import IDocument
from plone.app.contenttypes.interfaces import ILink
from plone.app.dexterity import textindexer
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.autoform import directives as form
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from zope.component import adapter
from zope.interface import implementer
from zope.interface import provider


class IArgomentiSchema(model.Schema):
    """Marker interface for Argomenti"""

    tassonomia_argomenti = RelationList(
        title=_("tassonomia_argomenti_label", default="Argomenti"),
        description=_(
            "tassonomia_argomenti_help",
            default="Seleziona una lista di argomenti d'interesse per questo"
            " contenuto.",
        ),
        value_type=RelationChoice(
            title=_("Argomenti correlati"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=False,
        default=[],
    )
    correlato_in_evidenza = RelationList(
        title=_("correlato_in_evidenza_label", default="Correlato in evidenza"),
        description=_(
            "correlato_in_evidenza_help",
            default="Seleziona un correlato da mettere in evidenza per questo"
            " contenuto.",
        ),
        value_type=RelationChoice(
            title=_("Correlato in evidenza"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=False,
        default=[],
    )

    form.widget(
        "tassonomia_argomenti",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Pagina Argomento"],
        },
    )
    form.widget(
        "correlato_in_evidenza",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={"maximumSelectionSize": 1},
    )

    textindexer.searchable("tassonomia_argomenti")


@provider(IFormFieldProvider)
class IArgomentiLink(model.Schema):
    """Marker interface for Argomenti"""

    tassonomia_argomenti = RelationList(
        title=_("tassonomia_argomenti_label", default="Argomenti"),
        description=_(
            "tassonomia_argomenti_help",
            default="Seleziona una lista di argomenti d'interesse per questo"
            " contenuto.",
        ),
        value_type=RelationChoice(
            title=_("Argomenti correlati"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=False,
        default=[],
    )

    form.widget(
        "tassonomia_argomenti",
        RelatedItemsFieldWidget,
        vocabulary="plone.app.vocabularies.Catalog",
        pattern_options={
            "selectableTypes": ["Pagina Argomento"],
        },
    )

    textindexer.searchable("tassonomia_argomenti")


@provider(IFormFieldProvider)
class IArgomenti(IArgomentiSchema):
    """ """

    model.fieldset(
        "correlati",
        label=_("correlati_label", default="Contenuti collegati"),
        fields=["correlato_in_evidenza"],
    )
    form.order_after(correlato_in_evidenza="IRelatedItems.relatedItems")


@provider(IFormFieldProvider)
class IArgomentiNews(IArgomentiSchema):
    """ """

    tassonomia_argomenti = RelationList(
        title=_("tassonomia_argomenti_label", default="Argomenti"),
        description=_(
            "tassonomia_argomenti_help",
            default="Seleziona una lista di argomenti d'interesse per questo"
            " contenuto.",
        ),
        value_type=RelationChoice(
            title=_("Argomenti correlati"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=True,
        default=[],
    )

    correlato_in_evidenza = RelationList(
        title=_("correlato_in_evidenza_label", default="Correlato in evidenza"),
        description=_(
            "correlato_in_evidenza_help",
            default="Seleziona un correlato da mettere in evidenza per questo"
            " contenuto.",
        ),
        value_type=RelationChoice(
            title=_("Correlato in evidenza"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=False,
        default=[],
    )

    model.fieldset(
        "correlati",
        label=_("correlati_label", default="Contenuti collegati"),
        fields=["correlato_in_evidenza"],
    )


@provider(IFormFieldProvider)
class IArgomentiEvento(IArgomentiSchema):
    """ """

    tassonomia_argomenti = RelationList(
        title=_("tassonomia_argomenti_label", default="Argomenti"),
        description=_(
            "tassonomia_argomenti_help",
            default="Seleziona una lista di argomenti d'interesse per questo"
            " contenuto.",
        ),
        value_type=RelationChoice(
            title=_("Argomenti correlati"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=True,
        default=[],
    )


@provider(IFormFieldProvider)
class IArgomentiServizio(IArgomentiSchema):
    tassonomia_argomenti = RelationList(
        title=_("tassonomia_argomenti_label", default="Argomenti"),
        description=_(
            "tassonomia_argomenti_help",
            default="Seleziona una lista di argomenti d'interesse per questo"
            " contenuto.",
        ),
        value_type=RelationChoice(
            title=_("Argomenti correlati"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=True,
        default=[],
    )


@provider(IFormFieldProvider)
class IArgomentiDocumento(IArgomentiSchema):
    """ """

    tassonomia_argomenti = RelationList(
        title=_("tassonomia_argomenti_label", default="Argomenti"),
        description=_(
            "tassonomia_argomenti_help",
            default="Seleziona una lista di argomenti d'interesse per questo"
            " contenuto.",
        ),
        value_type=RelationChoice(
            title=_("Argomenti correlati"),
            vocabulary="plone.app.vocabularies.Catalog",
        ),
        required=True,
        default=[],
    )

    model.fieldset(
        "correlati",
        label=_("correlati_label", default="Contenuti collegati"),
        fields=["correlato_in_evidenza"],
    )
    form.order_after(correlato_in_evidenza="IRelatedItems.relatedItems")
    form.order_after(tassonomia_argomenti="IDublinCore.title")


@provider(IFormFieldProvider)
class IArgomentiDocument(IArgomentiSchema):
    """ """

    model.fieldset(
        "correlati",
        label=_("correlati_label", default="Contenuti collegati"),
        fields=["correlato_in_evidenza"],
    )
    model.fieldset(
        "testata",
        label=_("testata_fieldset_label", default="Testata"),
        fields=["tassonomia_argomenti"],
    )
    form.order_after(correlato_in_evidenza="IRelatedItems.relatedItems")
    form.order_after(tassonomia_argomenti="IInfoTestata.mostra_navigazione")


@provider(IFormFieldProvider)
class IArgomentiBando(IArgomentiSchema):
    """ """

    model.fieldset(
        "correlati",
        label=_("correlati_label", default="Contenuti collegati"),
        fields=["tassonomia_argomenti", "correlato_in_evidenza"],
    )
    form.order_before(tassonomia_argomenti="IRelatedItems.relatedItems")
    form.order_after(correlato_in_evidenza="IRelatedItems.relatedItems")


@implementer(IArgomenti)
@adapter(IDexterityContent)
class Argomenti(object):
    """"""

    def __init__(self, context):
        self.context = context


@implementer(IArgomentiDocumento)
@adapter(IDocumento)
class ArgomentiDocumento(Argomenti):
    """"""


@implementer(IArgomentiBando)
@adapter(IBandoAgidSchema)
class ArgomentiBando(Argomenti):
    """"""


@implementer(IArgomentiDocument)
@adapter(IDocument)
class ArgomentiDocument(Argomenti):
    """"""

    def __init__(self, context):
        self.context = context


@implementer(IArgomentiServizio)
@adapter(IServizio)
class ArgomentiServizio(object):
    """"""

    def __init__(self, context):
        self.context = context


@implementer(IArgomentiNews)
@adapter(IServizio)
class ArgomentiNews(object):
    """"""

    def __init__(self, context):
        self.context = context


@implementer(IArgomentiEvento)
@adapter(IServizio)
class ArgomentiEvento(object):
    """"""

    def __init__(self, context):
        self.context = context


@implementer(IArgomentiLink)
@adapter(ILink)
class ArgomentiLink(object):
    """"""

    def __init__(self, context):
        self.context = context



from plone import api
from plone.restapi.behaviors import IBlocks
from plone.restapi.interfaces import IBlockFieldSerializationTransformer
from plone.restapi.types.utils import get_info_for_type
from zope.component import adapter
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.publisher.interfaces.browser import IBrowserRequest

import logging


logger = logging.getLogger(__name__)


@implementer(IBlockFieldSerializationTransformer)
@adapter(IBlocks, IBrowserRequest)
class SearchTableVariationBlockSerialize:
    order = 100
    block_type = "search"
    value_field = "listingBodyTemplate"

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def _get_schema(self, portal_type):
        dtool = queryMultiAdapter(
            (api.portal.get(), self.request), name="dexterity-types"
        )
        try:
            dtype = dtool.publishTraverse(self.request, portal_type)
        except KeyError:
            logger.warning(f"KeyError: portal_type '{portal_type}' not found.")
            return None
        schema = get_info_for_type(dtype, self.request, portal_type)
        return schema

    def __call__(self, value):
        schemas = {}
        if value.get(self.value_field) == "table":
            for col in value.get("columns") or []:
                if not col.get("ct") or not col.get("field"):
                    continue
                if col["ct"] not in schemas:
                    # get schema as plone.restapi /@types/Schema
                    schemas[col["ct"]] = self._get_schema(col["ct"])
                if not schemas[col["ct"]]:
                    continue
                schema = schemas[col["ct"]]
                if "properties" in schema and col["field"] in schema["properties"]:
                    # TODO: ma servono veramente tutte le info o solo una parte ?
                    col["field_properties"] = schema["properties"][col["field"]]
        return value


class ListingTableVariationBlockSerialize(SearchTableVariationBlockSerialize):
    block_type = "listing"
    value_field = "variation"
